package com.utic.center.utic.traf.server.dao.mapper.dwdb;

import com.utic.center.utic.traf.server.dto.LinkPtrnVal;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@Mapper
public interface LinkPatternMapper {

    List<LinkPtrnVal> findLinkPatternHoliday15M(@Param("ptrnTime") String ptrnTime, @Param("logDate") String logDate, @Param("linkLevel") String linkLevel);
    List<LinkPtrnVal> findLinkPatternSun15M(@Param("ptrnTime") String ptrnTime, @Param("logDate") String logDate, @Param("linkLevel") String linkLevel);
    List<LinkPtrnVal> findLinkPatternMon15M(@Param("ptrnTime") String ptrnTime, @Param("logDate") String logDate, @Param("linkLevel") String linkLevel);
    List<LinkPtrnVal> findLinkPatternFri15M(@Param("ptrnTime") String ptrnTime, @Param("logDate") String logDate, @Param("linkLevel") String linkLevel);
    List<LinkPtrnVal> findLinkPatternSat15M(@Param("ptrnTime") String ptrnTime, @Param("logDate") String logDate, @Param("linkLevel") String linkLevel);
    List<LinkPtrnVal> findLinkPatternWeek15M(@Param("ptrnTime") String ptrnTime, @Param("logDate") String logDate, @Param("linkLevel") String linkLevel);
}
